
/** @file
 *
 * @defgroup main.c
 * @{
 * @ingroup
 * @brief   SPI and GPIOTE test project 
 *
 * This file contains the source code for a sample application
 */

// - Standard
#include <stdint.h>
#include <string.h>
#include "sdk_config.h"
// - Nordic
#include "nordic_common.h"
#include "nrf.h"
#include "nrf_delay.h"
// - Nordic drivers
#include "nrf_drv_gpiote.h"
#include "nrf_drv_spi.h"
// - Nordic app
#include "app_timer.h"
#include "app_button.h"
#include "app_util_platform.h"
// - Softdevice
#include "softdevice_handler.h"
// - Hardware support
#include "boards.h"
#include "bsp.h"

#define NRF_LOG_MODULE_NAME "MAIN"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"


#define APP_TIMER_PRESCALER             0                                           /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_OP_QUEUE_SIZE         4                                           /**< Size of timer operation queues. */
#define DEAD_BEEF                       0xDEADBEEF

#define SPI_INSTANCE  1 /**< SPI instance index. */
static const nrf_drv_spi_t spi = NRF_DRV_SPI_INSTANCE(SPI_INSTANCE);  /**< SPI instance. */
static volatile bool spi_xfer_done;  /**< Flag used to indicate that SPI instance completed the transfer. */

#define TEST_STRING "Nordic"
static uint8_t       m_tx_buf[] = TEST_STRING;           /**< TX buffer. */
static uint8_t       m_rx_buf[sizeof(TEST_STRING) + 1];    /**< RX buffer. */
static const uint8_t m_length = sizeof(m_tx_buf);        /**< Transfer length. */
static bool          send_packet = false;

/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

/**@brief Function for placing the application in low power state while waiting for events.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}
/*
 * @brief        Interrupt handler for interrupt line of chip
 */
void gpiote_irq_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
    NRF_LOG_DEBUG("\r\n\r\nGPIOTE IRQ Handler called on PIN %d with action %d\r\n", pin, action);
}

/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
void bsp_event_handler(bsp_event_t event)
{
    NRF_LOG_DEBUG("Bsp event : %d\r\n", (uint32_t) event);
    switch (event)
    {
        case BSP_BUTTON_0:
        case BSP_BUTTON_1:
        case BSP_BUTTON_2:
        case BSP_BUTTON_3:
            send_packet = true;
            break;
        default:
            break;
    }
}
/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init()
{
    uint32_t err_code = bsp_init(BSP_INIT_LED | BSP_INIT_BUTTONS,
                                 APP_TIMER_TICKS(100, APP_TIMER_PRESCALER),
                                 bsp_event_handler);
    APP_ERROR_CHECK(err_code);

}

/**
 * @brief SPI user event handler.
 * @param event
 */
void spi_event_handler(nrf_drv_spi_evt_t const * p_event)
{
    spi_xfer_done = true;
    NRF_LOG_INFO("Transfer completed.\r\n");
    LEDS_INVERT(BSP_LED_1_MASK);
    if (m_rx_buf[0] != 0)
    {
        NRF_LOG_INFO(" Received: \r\n");
        NRF_LOG_HEXDUMP_INFO(m_rx_buf, strlen((const char *)m_rx_buf));
    }
}


/**@brief Application main function.
 */
int main(void)
{
    nrf_clock_lf_cfg_t clock_lf_cfg = NRF_CLOCK_LFCLKSRC;
    

    // Init GPIOTE driver
    if(!nrf_drv_gpiote_is_init())
    {
        APP_ERROR_CHECK(nrf_drv_gpiote_init());
    }
    APP_ERROR_CHECK(NRF_LOG_INIT(NULL));

    NRF_LOG_INFO("Start Testing project\r\n");
    // Initialize.
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);

    // Initialize SoftDevice.
    SOFTDEVICE_HANDLER_INIT(&clock_lf_cfg, NULL);
    // - Buttons and leds
    buttons_leds_init();

#define TEST_SPI    1
#define TEST_GPIOTE 1


#if TEST_SPI
    {
        nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
        spi_config.ss_pin   = 29;
        spi_config.miso_pin = 28;
        spi_config.mosi_pin = 4;
        spi_config.sck_pin  = 3;
        APP_ERROR_CHECK(nrf_drv_spi_init(&spi, &spi_config, spi_event_handler));
    }
#endif

#if TEST_GPIOTE

#define PIN_TEST    31
    NRF_LOG_INFO("Start gpiote driver\r\n");
    {
        // Configure interrupt as rising edge...
        nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(true);

        // ... and no pull action on the line
        in_config.pull = NRF_GPIO_PIN_PULLDOWN;
        //in_config.pull = NRF_GPIO_PIN_NOPULL;

        // Init GPIOTE with configuration on PINs
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(PIN_TEST, &in_config, gpiote_irq_handler));
        nrf_drv_gpiote_in_event_enable(PIN_TEST, true);
    }
#endif
    // Enter main loop.
    for (;;)
    {
        if(send_packet)
        {
            APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, m_tx_buf, m_length, m_rx_buf, m_length));
            send_packet = false;
        }
        LEDS_INVERT(BSP_LED_0);
        //NRF_SPI1->ENABLE = 0;
        NRF_LOG_DEBUG("EnterPWR MGT\r\n");
        power_manage();
        NRF_LOG_DEBUG("Exit PWR MGT\r\n");
    }
}


/**
 * @}
 */
